@@ -1,5 +1,5 @@
 # 
-# Copyright (C) 2010 Platform Computing
+# Copyright (C) 2010-2011 Platform Computing
 # 
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -28,7 +28,7 @@ Created on Nov 10, 2010
 from pyocci import registry, service
 from pyocci.examples.keyvalue import KeyValueBackend, KeyValueLink
 from pyocci.rendering_parsers import TextPlainRendering, TextHeaderRendering, \
-    TextHTMLRendering
+    TextHTMLRendering, URIListRendering
 from pyocci.service import ResourceHandler, ListHandler, QueryHandler, \
     LoginHandler, LogoutHandler
 import tornado.httpserver
@@ -84,6 +84,7 @@ if __name__ == '__main__':
     registry.register_parser(TextHTMLRendering.content_type,
                                        HTML_RENDERER)
     registry.register_parser('application/x-www-form-urlencoded', HTML_RENDERER)
+    registry.register_parser(URIListRendering.content_type, URIListRendering())
 
     # register a simple key value backend
     registry.register_backend([KeyValueBackend.kind,
