@@ -139,22 +139,19 @@ class Rendering(object):
         raise NotImplementedError()
 
 #===============================================================================
-# Convenient routines
+# Convenient routines for the syntax defined in the spec
 #===============================================================================
 
-def _get_category(cat_string):
+def _from_http_category(category_string):
     '''
     Create a category instance from an string.
-    
-    @param cat_string: The string with RFC compliant syntac of a Category.
-    @type cat_string: str
+
+    @param category_string: The string with RFC compliant syntac of a Category.
+    @type category_string: str
     '''
-
-    # TODO: make use of the class attribute
-
     cat = Category()
     # find the term
-    tmp = cat_string.split(';')
+    tmp = category_string.split(';')
     term = tmp[0].strip()
     if re.match("^[\w\d_-]*$", term) and term is not '':
         cat.term = term.strip()
@@ -163,13 +160,13 @@ def _get_category(cat_string):
                                + ' be found.')
 
     # find the scheme
-    begin = cat_string.find('scheme=')
+    begin = category_string.find('scheme=')
     if begin is not - 1:
-        tmp = cat_string[begin + 7:]
+        tmp = category_string[begin + 7:]
         end = tmp.find(";")
         if end > -1:
             tmp = tmp[:end]
-        scheme = tmp.rstrip('"').lstrip('"')
+        scheme = _strip_all(tmp)
         if scheme.find('http') is not - 1:
             cat.scheme = scheme.strip().rstrip('#')
         else:
@@ -177,9 +174,9 @@ def _get_category(cat_string):
                                    + ' could be found.')
 
     # find the location
-    begin = cat_string.find('location=')
+    begin = category_string.find('location=')
     if begin is not - 1:
-        tmp = cat_string[begin + 9:]
+        tmp = category_string[begin + 9:]
         end = tmp.find(";")
         if end > -1:
             tmp = tmp[:end]
@@ -194,6 +191,153 @@ def _get_category(cat_string):
 
     return cat
 
+def _to_http_category(kind, extended = False):
+    '''
+    Create a category rendering for a kind or mixin. If extended it will try to
+    put in as much information as possible.
+
+    @param kind: A kind of a mixin.
+    @type kind: Kind or Mixin
+    @param extended: Indicating wether rendering should be minimal or not.
+    @type extended: boolean
+    '''
+    tmp = ''
+    tmp += kind.term
+    tmp += '; scheme="' + kind.scheme + '#"'
+    tmp += '; class="' + kind.cls_str + '"'
+    if extended is True:
+        if hasattr(kind, 'title') and kind.title is not '':
+            tmp += '; title="' + kind.title + '"'
+        if hasattr(kind, 'related') and len(kind.related) > 0:
+            rel_list = []
+            for item in kind.related:
+                rel_list.append(repr(item))
+            tmp += '; rel=' + ' '.join(rel_list)
+        if hasattr(kind, 'location') and kind.location is not '':
+            tmp += '; location=' + kind.location
+        if hasattr(kind, 'attributes') and len(kind.attributes) > 0:
+            tmp += '; attributes="' + ' '.join(kind.attributes) + '"'
+        if hasattr(kind, 'actions') and len(kind.actions) > 0:
+            action_list = []
+            for item in kind.actions:
+                action_list.append(repr(item))
+            tmp += '; actions="' + ' '.join(action_list) + '"'
+    return tmp
+
+def _from_http_link(link_string):
+    '''
+    Retrieve information rendered from HTTP Link definition.
+    '''
+    #target
+    tmp = link_string[:link_string.find('>')].strip()
+    target = tmp[1:]
+
+    # category...
+    begin = link_string.find('category=')
+    if begin is not - 1:
+        tmp = link_string[begin + 9:]
+        end = tmp.find(";")
+        if end > -1:
+            tmp = tmp[:end]
+        cat = _strip_all(tmp).split('#')
+        try:
+            term = cat[1]
+            scheme = cat[0]
+        except:
+            raise ParsingException('Syntax of the category definition seems'
+                                   + ' incorrect.')
+        else:
+            category = Category()
+            category.term = term
+            category.scheme = scheme
+    else:
+        raise ParsingException('Could not find a proper category definition')
+
+    # attributes...
+    # TODO: use partition
+    tmp = link_string[begin + 12 + len(repr(category)):]
+    attributes = {}
+    for item in tmp.rstrip(';').split(';'):
+        key, value = _from_http_attribute(item)
+        attributes[key] = value
+
+    result = Link()
+    result.kind = category
+    result.attributes = attributes
+    result.target = target
+    result.source = '#'
+    return result
+
+def _to_http_link(entity, action = None, is_action = False):
+    '''
+    Creates a HTTP Link rendering.
+    '''
+    if is_action:
+        string = '<' + registry.HOST + entity.identifier + '?action='
+        string += action.kind.term + '>; rel="' + str(action) + '"'
+        return string
+    else:
+        link_rendering = '<' + registry.HOST + entity.target + '>; '
+        link_rendering += 'rel="' + repr(service.RESOURCES[entity.target].kind)
+        link_rendering += '"; self="'
+        link_rendering += registry.HOST + entity.identifier + '"; '
+        link_rendering += 'category="' + repr(entity.kind) + '"; '
+
+        for attr in entity.attributes.keys():
+            link_rendering += attr + '="' + entity.attributes[attr]
+            link_rendering += '";'
+        return link_rendering
+
+def _from_http_attribute(attribute_string):
+    '''
+    Retrieve the attributes from the HTTP X - OCCI - Attribute rendering.
+    '''
+    try:
+        tmp = _strip_all(attribute_string)
+        key = tmp.split('=')[0]
+        value = tmp.split('=')[1]
+        if value.find('"') is not - 1:
+            value = _strip_all(value)
+    except IndexError:
+        raise ParsingException('Could not determine the given attributes')
+
+    return key, value
+
+def _to_http_attribute(key, value):
+    '''
+    Creates a HTTP X-OCCI-Attribute rendering.
+    '''
+    if value.find(' ') == -1:
+        return key + '=' + value
+    else:
+        return key + '="' + value + '"'
+
+def _from_http_location(location_string):
+    '''
+    Retrieve the informations from HTTP X-OCCI-Location rendering.
+    '''
+    if location_string.strip().find(registry.HOST) == 0:
+        identifier = location_string.strip()[len(registry.HOST):]
+    else:
+        identifier = location_string.strip()
+    return identifier
+
+def _to_http_location(item):
+    '''
+    Creates a HTTP X - OCCI - Location rendering.
+    '''
+    return registry.HOST + item
+
+def _strip_all(string):
+    '''
+    Removes beginning / ending quotes and whitespaces.
+    '''
+    return string.lstrip().lstrip('"').rstrip().rstrip('"')
+
+#===============================================================================
+# Convenient routines
+#===============================================================================
+
 def _get_categories(category_string_list):
     '''
     Retrieve the Kind and a list of categories. Will only return those which are
@@ -207,7 +351,9 @@ def _get_categories(category_string_list):
     categories = []
     for tmp in category_string_list:
         for cat_string in tmp.split(','):
-            cat = _get_category(cat_string)
+            cat = _from_http_category(cat_string)
+
+            # TODO: make use of the class attribute
 
             # now that we have a category try to look it up and use that objects
             for category in registry.BACKENDS.keys():
@@ -226,80 +372,63 @@ def _get_categories(category_string_list):
 
     return kind, categories
 
-def _get_attributes(attributes):
+def _to_link(link_string_list):
     '''
-    Parse the Attributes.
+    Will create a link resource instance.
     
-    @param attributes: List of attributes
-    @type attributes: list 
+    @param link_string_list: A list of link renderings
+    @type link_string_list: list 
     '''
-    tmp = {}
-    for item in attributes:
-        for attr in item.split(','):
-            try:
-                tmp[attr.split('=')[0].strip()] = attr.split('=')[1]
-            except IndexError:
-                raise ParsingException('Could not determine the attributes...')
-    return tmp
+    links = []
+    for item in link_string_list:
+        found = False
+        link = _from_http_link(item)
+        for category in registry.BACKENDS.keys():
+            if link.kind == category:
+                link.kind = category
+                links.append(link)
+                found = True
+                break
 
-def _from_category(kind, extended = False):
-    '''
-    Create a category rendering for a kind or mixin. If extended it will try to
-    put in as much information as possible.
-    
-    @param kind: A kind of a mixin.
-    @type kind: Kind or Mixin
-    @param extended: Indicating wether rendering should be minimal or not.
-    @type extended: boolean  
-    '''
-    tmp = ''
-    tmp += kind.term
-    tmp += ';scheme="' + kind.scheme + '#"'
-    tmp += ';class="' + kind.cls_str + '"'
-    if extended:
-        if hasattr(kind, 'title') and kind.title is not '':
-            tmp += ';title="' + kind.title + '"'
-        if hasattr(kind, 'related') and len(kind.related) > 0:
-            rel_list = []
-            for item in kind.related:
-                rel_list.append(repr(item))
-            tmp += ';rel=' + ' '.join(rel_list)
-        if hasattr(kind, 'location') and kind.location is not '':
-            tmp += ';location=' + kind.location
-        if hasattr(kind, 'attributes') and len(kind.attributes) > 0:
-            tmp += ';attributes="' + ' '.join(kind.attributes) + '"'
-        if hasattr(kind, 'actions') and len(kind.actions) > 0:
-            action_list = []
-            for item in kind.actions:
-                action_list.append(repr(item))
-            tmp += ';actions="' + ' '.join(action_list) + '"'
-    return tmp
+        if not found:
+            raise ParsingException('The given category for a link definition'
+                                   + ' was not registered in this service: '
+                                   + repr(link.kind))
+    return links
 
-def _to_entity(defined_kind, kind, categories, attributes, allow_incomplete):
+def _to_entity(defined_kind, data, allow_incomplete):
     '''
     Helper routine for creating a new entity.
     
     @param defined_kind: A previsously defined kind.
     @type defined_kind: Kind
-    @param kind: A kind
-    @type kind: Kind
-    @param categories: A list of all categories.
-    @type categories: list
-    @param attributes: A list of all attributes.
-    @type attributes: list
+    @param data: A HTTPData object.
+    @type data: HTTPData
     @param allow_incomplete: Indicates wether rendering should be complete.
     @type allow_incomplete: boolean
     '''
+
+    attributes = {}
+    for item in data.attributes:
+        key, value = _from_http_attribute(item)
+        attributes[key] = value
+
+    links = _to_link(data.links)
+
+    kind, categories = _get_categories(data.categories)
     if allow_incomplete:
         kind = defined_kind
-    elif kind is None:
-        raise ParsingException('Could not find a Kind for this request.')
+    if kind is None:
+        raise ParsingException('Could not determine a kind.')
 
     if Resource.category in kind.related:
         entity = Resource()
         if 'summary' in attributes.keys():
             entity.summary = attributes['summary']
             attributes.pop('summary')
+        if 'title' in attributes.keys():
+            entity.title = attributes['title']
+            attributes.pop('title')
     elif Link.category in kind.related:
         entity = Link()
         if 'source' in attributes.keys():
@@ -319,7 +448,7 @@ def _to_entity(defined_kind, kind, categories, attributes, allow_incomplete):
     entity.kind = kind
     entity.mixins = categories
     entity.attributes = attributes
-    return entity
+    return entity, links
 
 def _from_entity(entity):
     '''
@@ -331,38 +460,45 @@ def _from_entity(entity):
     result = HTTPData()
 
     # add kind
-    result.categories.append(_from_category(entity.kind))
+    result.categories.append(_to_http_category(entity.kind))
     # mixins...
     for item in entity.mixins:
-        result.categories.append(_from_category(item))
+        result.categories.append(_to_http_category(item))
 
     if isinstance(entity, Resource):
         # check if summary is available...
         if entity.summary is not '':
-            result.attributes.append('summary=' + entity.summary)
+            result.attributes.append(_to_http_attribute('summary',
+                                                        entity.summary))
+        if entity.title is not '':
+            result.attributes.append(_to_http_attribute('title',
+                                                        entity.title))
 
         if len(entity.actions) > 0:
             for action in entity.actions:
-                result.links.append('<' + registry.HOST + entity.identifier
-                                    + '?action=' + action.kind.term + '>')
+                result.links.append(_to_http_link(entity, action = action,
+                                                  is_action = True))
 
         if len(entity.links) > 0:
             for item in entity.links:
-                result.links.append('<' + registry.HOST + item.target
-                                    + '>;self=' + registry.HOST
-                                    + item.identifier + ';')
+                result.links.append(_to_http_link(item))
 
     elif isinstance(entity, Link):
         # source and target must be there!
-        result.attributes.append('source=' + registry.HOST + entity.source)
-        result.attributes.append('target=' + registry.HOST + entity.target)
+        result.attributes.append(_to_http_attribute('source',
+                                                    registry.HOST
+                                                    + entity.source))
+        result.attributes.append(_to_http_attribute('target',
+                                                    registry.HOST
+                                                    + entity.target))
     for item in entity.attributes.keys():
-        result.attributes.append(item + '=' + entity.attributes[item])
+        result.attributes.append(_to_http_attribute(item,
+                                                    entity.attributes[item]))
 
     return result
 
 #===============================================================================
-# Renderings for different content types can be found below this comment
+# text/plain, text/occi, text/uri-list and text/html rendering parsers
 #===============================================================================
 
 class TextPlainRendering(Rendering):
@@ -381,24 +517,45 @@ class TextPlainRendering(Rendering):
     def _get_data(self, body):
         '''
         Simple method to split out the information from the HTTP body.
-        
+
         @param body: The HTML body.
         @type body: str
         '''
         data = HTTPData()
         for entry in body.split('\n'):
             if entry.find('Category:') > -1:
-                data.categories.append(entry[entry.find('Category:') + 9:])
+                tmp = entry[entry.find('Category:') + 9:]
+                if tmp.find(',') == -1:
+                    data.categories.append(tmp)
+                else:
+                    for item in tmp.split(','):
+                        data.categories.append(item)
             if entry.find('X-OCCI-Attribute:') > -1:
-                data.attributes.append(entry[entry.find('X-OCCI-Attribute:')
-                                             + 17:])
+                tmp = entry[entry.find('X-OCCI-Attribute:') + 17:]
+                if tmp.find(',') == -1:
+                    data.attributes.append(tmp)
+                else:
+                    for item in tmp.split(','):
+                        data.attributes.append(item)
+            if entry.find('Link:') > -1:
+                tmp = entry[entry.find('Link:') + 5:]
+                if tmp.find(',') == -1:
+                    data.links.append(tmp)
+                    # TODO: handle multiple link renderings...
+            if entry.find('X-OCCI-Location:') > -1:
+                tmp = entry[entry.find('X-OCCI-Location:') + 16:]
+                if tmp.find(',') == -1:
+                    data.locations.append(tmp)
+                else:
+                    for item in tmp.split(','):
+                        data.locations.append(item)
         return data
 
     def from_categories(self, categories):
         headers = {}
         body = ''
         for item in categories:
-            body += 'Category: ' + _from_category(item, extended = True)
+            body += 'Category: ' + _to_http_category(item, extended = True)
             body += '\n'
         headers['Content-Type'] = self.content_type
         return headers, body
@@ -407,7 +564,8 @@ class TextPlainRendering(Rendering):
         headers = {}
         body = ''
         for item in entities:
-            body += 'X-OCCI-Location: ' + registry.HOST + item.identifier + '\n'
+            body += 'X-OCCI-Location: ' + _to_http_location(item.identifier)
+            body += '\n'
         headers['Content-Type'] = self.content_type
         return headers, body
 
@@ -428,15 +586,11 @@ class TextPlainRendering(Rendering):
         return headers, body
 
     def get_entities(self, headers, body):
+        data = self._get_data(body)
         ids = []
-        for entry in body.split('\n'):
-            if entry.find('X-OCCI-Location:') > -1:
-                tmp = entry[entry.find('X-OCCI-Location:') + 16:]
-                for item in tmp.split(','):
-                    if item.strip().find(registry.HOST) == 0:
-                        ids.append(item.strip()[len(registry.HOST):])
-                    else:
-                        ids.append(item.strip())
+        for item in data.locations:
+            ids.append(_from_http_location(item))
+
         if len(ids) > 0:
             return ids
         else:
@@ -444,11 +598,11 @@ class TextPlainRendering(Rendering):
                                    + ' pointing to resource instances.')
 
     def login_information(self):
-        html = 'Please do a POST operation with a name and pass attribute.'
+        text = 'Please do a POST operation with a name and pass attribute.'
 
         headers = {}
         headers['Content-Type'] = self.content_type
-        return headers, html
+        return headers, text
 
     def to_action(self, headers, body):
         data = self._get_data(body)
@@ -464,18 +618,20 @@ class TextPlainRendering(Rendering):
             for cat in categories:
                 action = Action()
                 action.kind = cat
-                action.attributes = _get_attributes(data.attributes)
+                for attr in data.attributes:
+                    key, value = _from_http_attribute(attr)
+                    action.attributes[key] = value
                 break
 
         del(data)
         return action
 
     def to_categories(self, headers, body):
+        data = self._get_data(body)
         categories = []
-        for entry in body.split('\n'):
-            if entry.find('Category:') > -1:
-                for item in entry[entry.find('Category:') + 9:].split(','):
-                    categories.append(_get_category(item))
+        for item in data.categories:
+            categories.append(_from_http_category(item))
+
         if len(categories) > 0:
             return categories
         else:
@@ -491,15 +647,10 @@ class TextPlainRendering(Rendering):
 
         data = self._get_data(body)
 
-        # Parse the data
-        kind, categories = _get_categories(data.categories)
-        attributes = _get_attributes(data.attributes)
-
-        entity = _to_entity(defined_kind, kind, categories, attributes,
-                            allow_incomplete)
+        entity, links = _to_entity(defined_kind, data, allow_incomplete)
 
         del(data)
-        return entity
+        return entity, links
 
 class TextHeaderRendering(Rendering):
     '''
@@ -518,23 +669,28 @@ class TextHeaderRendering(Rendering):
     def _get_data(self, headers):
         '''
         Simple method to split out the information from the HTTP headers.
-        
+
         @param headers: The HTML headers.
         @type headers: dict
         '''
         # split out the information
         data = HTTPData()
         if 'Category' in headers.keys():
-            data.categories.append(headers['Category'])
+            data.categories = headers['Category'].split(',')
         if 'X-OCCI-Attribute' in headers.keys():
-            data.attributes.append(headers['X-OCCI-Attribute'])
+            data.attributes = headers['X-OCCI-Attribute'].split(',')
+        if 'X-OCCI-Location' in headers.keys():
+            data.locations = headers['X-OCCI-Location'].split(',')
+        if 'Link' in headers.keys():
+            data.links = headers['Link'].split(',')
         return data
 
     def from_categories(self, categories):
         headers = {}
         tmp = []
         for item in categories:
-            tmp.append(_from_category(item, extended = True))
+            tmp.append(_to_http_category(item, extended = True))
+
         if len(tmp) > 0:
             headers['Category'] = ','.join(tmp)
         headers['Content-Type'] = self.content_type
@@ -544,7 +700,7 @@ class TextHeaderRendering(Rendering):
         headers = {}
         tmp = []
         for item in entities:
-            tmp.append(registry.HOST + item.identifier)
+            tmp.append(_to_http_location(item.identifier))
         if len(tmp) > 0:
             headers['X-OCCI-Location'] = ','.join(tmp)
         headers['Content-Type'] = self.content_type
@@ -566,31 +722,28 @@ class TextHeaderRendering(Rendering):
         return headers, 'OK'
 
     def get_entities(self, headers, body):
+        data = self._get_data(headers)
         ids = []
-        if 'X-OCCI-Location' in headers.keys():
-            for item in headers['X-OCCI-Location'].split(','):
-                if item.strip().find(registry.HOST) == 0:
-                    ids.append(item.strip()[len(registry.HOST):])
-                else:
-                    ids.append(item.strip())
+        for item in data.locations:
+            ids.append(_from_http_location(item))
+
         if len(ids) > 0:
             return ids
         else:
             raise ParsingException('Header does not contain a location.')
 
     def login_information(self):
-        html = 'Please do a POST operation with a name and pass attribute.'
+        text = 'Please do a POST operation with a name and pass attribute.'
 
         headers = {}
         headers['Content-Type'] = self.content_type
-        return headers, html
+        return headers, text
 
     def to_action(self, headers, body):
         data = self._get_data(headers)
 
         # all data...
         kind, categories = _get_categories(data.categories)
-        attributes = _get_attributes(data.attributes)
 
         action = None
         if len(categories) == 0:
@@ -600,17 +753,21 @@ class TextHeaderRendering(Rendering):
             for cat in categories:
                 action = Action()
                 action.kind = cat
-                action.attributes = _get_attributes(data.attributes)
+                for attr in data.attributes:
+                    key, value = _from_http_attribute(attr)
+                    action.attributes[key] = value
                 break
 
         del(data)
         return action
 
     def to_categories(self, headers, body):
+        data = self._get_data(headers)
+
         categories = []
-        if 'Category' in headers.keys():
-            for item in headers['Category'].split(','):
-                categories.append(_get_category(item))
+        for item in data.categories:
+            categories.append(_from_http_category(item))
+
         if len(categories) > 0:
             return categories
         else:
@@ -625,19 +782,14 @@ class TextHeaderRendering(Rendering):
         # split out the information
         data = self._get_data(headers)
 
-        # all data...
-        kind, categories = _get_categories(data.categories)
-        attributes = _get_attributes(data.attributes)
-
-        entity = _to_entity(defined_kind, kind, categories, attributes,
-                            allow_incomplete)
+        entity, links = _to_entity(defined_kind, data, allow_incomplete)
 
         del(data)
-        return entity
+        return entity, links
 
 class URIListRendering(Rendering):
     '''
-    This rendering can handle text/uri-list requests. Cannot be used for
+    This rendering can handle text / uri - list requests. Cannot be used for
     creation of entities or similar.
     '''
 
@@ -710,7 +862,7 @@ class TextHTMLRendering(Rendering):
     def _get_data(self, body):
         '''
         Simple method to split out the information from the HTTP HTML body.
-        
+
         @param headers: The HTML body.
         @type headers: str
         '''
@@ -913,39 +1065,10 @@ class TextHTMLRendering(Rendering):
 
         data = self._get_data(body)
 
-        # all data...
-        kind, categories = _get_categories(data.categories)
-        attributes = _get_attributes(data.attributes)
-
-        if allow_incomplete:
-            kind = defined_kind
-
-        if Resource.category in kind.related:
-            entity = Resource()
-            if 'summary' in attributes.keys():
-                entity.summary = attributes['summary']
-                attributes.pop('summary')
-        elif Link.category in kind.related:
-            entity = Link()
-            if 'source' in attributes.keys():
-                source = attributes['source']
-                if source.find(registry.HOST) == 0:
-                    entity.source = source[len(registry.HOST):]
-                else:
-                    entity.source = attributes['source']
-                attributes.pop('source')
-            if 'target' in attributes.keys():
-                target = attributes['target']
-                if target.find(registry.HOST) == 0:
-                    entity.target = target[len(registry.HOST):]
-                else:
-                    entity.target = target
-                attributes.pop('target')
-        entity.kind = kind
-        entity.attributes = attributes
+        entity, links = _to_entity(defined_kind, data, allow_incomplete)
 
         del(data)
-        return entity
+        return entity, links
 
     #===========================================================================
     # Some routines for convenience
@@ -964,7 +1087,7 @@ class TextHTMLRendering(Rendering):
         tmp += '<tr><th>Class</th><td>' + category.cls_str + '</td></tr>'
 
         if hasattr(category, 'title') and category.title is not '':
-            tmp += '<tr><th>Title/th><td>' + category.title + '</td></tr>'
+            tmp += '<tr><th>Title</th><td>' + category.title + '</td></tr>'
         if hasattr(category, 'actions') and len(category.actions) > 0:
             tmp += '<tr><th>Actions</th><td><ul>'
             for item in category.actions:
