@@ -69,6 +69,8 @@ class BaseHandler(tornado.web.RequestHandler):
             heads['X-OCCI-Attribute'] = headers['X-Occi-Attribute']
         if 'X-Occi-Location' in headers:
             heads['X-OCCI-Location'] = headers['X-Occi-Location']
+        if 'Link' in headers:
+            heads['Link'] = headers['Link']
         if self.request.body is not '':
             body = self.request.body.strip()
         else:
@@ -164,7 +166,6 @@ class ResourceHandler(BaseHandler):
     #                                               (methods exists twice...)
     # pylint: disable=R0904,W0221
 
-    # XXX: What to do with links when creating a resource?
     @tornado.web.authenticated
     def post(self, key):
         headers, body = self.extract_http_data()
@@ -183,15 +184,29 @@ class ResourceHandler(BaseHandler):
             # parse the request
             entity = None
             try:
-                entity = parser.to_entity(headers, body)
+                # create the entity
+                entity, links = parser.to_entity(headers, body)
+                entity.owner = self.get_current_user()
+                key = self._create_key(entity)
                 backend = registry.get_backend(entity.kind)
                 backend.create(entity)
+
+                RESOURCES[key] = entity
+
+                # handle the links...
+                for item in links:
+                    backend = registry.get_backend(item.kind)
+                    item.source = key
+                    backend.create(item)
+                    key = self._create_key(item)
+                    RESOURCES[key] = item
+                    item.idetifier = key
+                    item.owner = self.get_current_user()
+
             except (ParsingException, AttributeError) as pse:
+                # TODO: RESOURCES.pop(key)
                 raise HTTPError(400, str(pse))
 
-            entity.owner = self.get_current_user()
-            key = self._create_key(entity)
-            RESOURCES[key] = entity
         self._send_response({'Location': registry.HOST + key}, 'OK')
 
     @tornado.web.authenticated
@@ -203,7 +218,7 @@ class ResourceHandler(BaseHandler):
             old_entity = RESOURCES[key]
             new_entity = None
             try:
-                new_entity = parser.to_entity(headers, body,
+                new_entity, links = parser.to_entity(headers, body,
                                               allow_incomplete = True,
                                               defined_kind = old_entity.kind)
 
@@ -217,15 +232,29 @@ class ResourceHandler(BaseHandler):
                 raise HTTPError(400, str(pse))
         else:
             try:
-                new_entity = parser.to_entity(headers, body)
+                new_entity, links = parser.to_entity(headers, body)
+                new_entity.identifier = key
+                new_entity.owner = self.get_current_user()
 
                 backend = registry.get_backend(new_entity.kind)
                 backend.create(new_entity)
+
+                RESOURCES[key] = new_entity
+
+                # handle the links...
+                for item in links:
+                    backend = registry.get_backend(item.kind)
+                    item.source = key
+                    backend.create(item)
+                    key = self._create_key(item)
+                    RESOURCES[key] = item
+                    item.idetifier = key
+                    item.owner = self.get_current_user()
+
             except (ParsingException, AttributeError) as pse:
+                # TODO: RESOURCES.pop(key)
                 raise HTTPError(400, str(pse))
-            RESOURCES[key] = new_entity
-            new_entity.identifier = key
-            new_entity.owner = self.get_current_user()
+
         self._send_response(None, 'OK')
 
     @tornado.web.authenticated
@@ -418,9 +447,6 @@ class QueryHandler(BaseHandler):
     This class represents the OCCI query interface.
     '''
 
-    # TODO: check for adding one location twice...
-    # TODO: filter categories...
-
     # disabling 'Too many public methods' pylint check (tornado's fault)
     # pylint: disable=R0904
 
@@ -589,11 +615,12 @@ class LinkBackend(Backend):
             raise AttributeError('A link needs to have a target.')
 
         try:
+            RESOURCES.has_key(link.target)
             src = RESOURCES[link.source]
-
             src.links.append(link)
-        except KeyError:
-            raise AttributeError('Source and target need to be valid Resources')
+        except KeyError as notfound:
+            raise AttributeError('Source and target need to be valid'
+                                 + ' Resources: ' + str(notfound))
 
     def retrieve(self, link):
         pass
