@@ -15,7 +15,7 @@
 # 
 # You should have received a copy of the GNU Lesser General Public
 # License along with this library; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301 USA
 # 
 from distutils.core import setup, Extension
 
@@ -26,12 +26,14 @@ setup(name = 'pyssf',
       keywords = 'REST,RESTful,OCCI,LSF,Platform,Cloud,Grid,Cluster,HPC',
       url = 'http://pyssf.sourceforge.net',
       packages = ['ssf', 'pyocci', 'pydrmaa'],
-      long_description = 'A set of Python modules to integrate you existing or future applications with Clouds, Grids and Clusters.',
+      long_description = 'A set of Python modules to integrate you existing or'
+      + ' future applications with Clouds, Grids and Clusters.',
       ext_package = 'pylsf',
       ext_modules = [Extension('_lsf', ['pylsf/lsf.i'],
                                include_dirs = ['/usr/include/python2.6'],
                                library_dirs = ['/opt/lsf/7.0/linux2.6-glibc2.3-x86_64/lib'],
-                               libraries = ['c', 'nsl', 'lsf', 'bat', 'fairshareadjust', 'lsbstream'])],
+                               libraries = ['c', 'nsl', 'lsf', 'bat',
+                                            'fairshareadjust', 'lsbstream'])],
       py_modules = ['pylsf.lsf'],
       classifiers = ["Development Status :: 2 - Pre-Alpha",
                      "License :: OSI Approved :: GNU Library or Lesser General Public License (LGPL)",
